API_DATA = b"Q0sJVEtPDWt7bmg2YnhtY344enZlMSRsPjxtaTI7P2Q0PTRhNWc="
API_DATA_OAUTH = b"eW91c3V3dn9mYjF2az48bzM7Pm4wZzdlZzpk"

NETWORK_ERROR_RETRY_COUNT = 50
OAUTH_LOCAL_PORT = 10500
SYNC_CHUNK_MAX_RESULTS = 200
SYNC_MAX_DOWNLOAD_WORKERS = 5
CURRENT_DB_VERSION = 3
